/**

  @file
  
  @brief implementation of the Aho-Corasick pattern matching algorithm.
    
  most of this code was taken from another module by Georgios Portokalidis
  with his permission. Since I don't really know how he has implemented the code
  I haven't tried to document it. 
  
  ----
  
  Fairly Fast Packet Filter
  http://ffpf.sourceforge.net/

  Copyright (c), 2003 - 2004 Georgios Portokalidis, Herbert Bos & Willem de Bruijn
  contact info : wdebruij_AT_users.sourceforge.net

  ----
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef AHO_CORASICK_H
#define AHO_CORASICK_H

typedef unsigned int aho_corasick_int_t;

#define AHO_CORASICK_CHARACTERS 256

struct aho_corasick_state
{
	aho_corasick_int_t id;
	aho_corasick_int_t depth;
	size_t output;
	struct aho_corasick_state *fail;
	struct aho_corasick_state *transitions[AHO_CORASICK_CHARACTERS];
};

typedef struct aho_corasick_state aho_corasick_state_t;

struct aho_corasick
{
	aho_corasick_int_t newstate;
	aho_corasick_state_t *zerostate;
};

typedef struct aho_corasick aho_corasick_t;

void aho_corasick_init(aho_corasick_t *);
int aho_corasick_addstring(aho_corasick_t *,unsigned char *);
int aho_corasick_maketree(aho_corasick_t *);

aho_corasick_int_t ahocorasick_KeywordTree_search_helper(aho_corasick_t *,unsigned char *,size_t, size_t *, size_t *);
aho_corasick_int_t ahocorasick_KeywordTree_search_long_helper(aho_corasick_t *,unsigned char *,size_t, size_t *, size_t *);

void aho_corasick_destroy(aho_corasick_t *);


#endif
